"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAxiosClient = exports.AxiosClientContext = void 0;
const equal_1 = require("@acrool/js-utils/equal");
const react_locale_1 = require("@acrool/react-locale");
const react_1 = __importStar(require("react"));
const react_router_dom_1 = require("react-router-dom");
const exception_1 = require("@/exception");
const redux_1 = require("@/library/redux");
const auth_1 = require("@/store/main/auth");
const utils_1 = require("@/store/main/auth/utils");
const AxiosCancelException_1 = __importDefault(require("./AxiosCancelException"));
const config_1 = require("./config");
const utils_2 = require("./utils");
let pendingRequestQueues = [];
let isTokenRefreshing = false;
exports.AxiosClientContext = (0, react_1.createContext)(config_1.axiosInstance);
const useAxiosClient = () => (0, react_1.useContext)(exports.AxiosClientContext);
exports.useAxiosClient = useAxiosClient;
const AxiosClientProvider = ({ children, }) => {
    const { locale, i18n } = (0, react_locale_1.useLocale)();
    const RefreshToken = auth_1.authHook.useRefreshToken();
    const KitOut = auth_1.authHook.useKickOut();
    const navigate = (0, react_router_dom_1.useNavigate)();
    const dispatch = (0, redux_1.useDispatch)();
    (0, react_1.useLayoutEffect)(() => {
        // 處理請求回應攔截
        const interceptorReq = config_1.axiosInstance.interceptors.request.use(interceptorsRequest);
        const interceptorRes = config_1.axiosInstance.interceptors.response.use(interceptorsResponse);
        return () => {
            config_1.axiosInstance.interceptors.request.eject(interceptorReq);
            config_1.axiosInstance.interceptors.response.eject(interceptorRes);
        };
    }, [isTokenRefreshing]);
    /**
     * 發起 Token Refresh
     */
    const postRefreshToken = () => {
        RefreshToken.mutate(undefined, {
            onSuccess: data => {
                runPendingRequest(true);
            },
            onError: data => {
                runPendingRequest(false);
            }
        });
    };
    /**
     * 執行所有貯列
     * @param isSuccess 是否成功
     */
    const runPendingRequest = (isSuccess) => {
        isTokenRefreshing = false; // 注意這個順序
        for (const cb of pendingRequestQueues) {
            cb(isSuccess);
        }
        pendingRequestQueues = [];
    };
    /**
     * 新增項目到貯列;
     * @param resolve
     * @param reject
     */
    const pushPendingRequestQueues = (resolve, reject) => {
        return (originConfig) => {
            pendingRequestQueues.push((isTokenRefreshOK) => {
                if (isTokenRefreshOK) {
                    resolve((0, config_1.axiosInstance)(originConfig));
                }
                else {
                    reject(new exception_1.SystemException({
                        message: i18n('errorHttp.401', { def: '請求的API沒有權限' }),
                        code: 'SERVICE_HTTP_401',
                    }));
                }
            });
        };
    };
    /**
     * 請求攔截器
     * @param originConfig
     */
    const interceptorsRequest = (originConfig) => {
        return new Promise((resolve, reject) => {
            // 設定 Common Header
            const { accessToken } = (0, utils_1.getTokenInfo)();
            // console.log('accessToken', accessToken);
            originConfig.headers['Accept-Language'] = locale;
            originConfig.headers['Authorization'] = (0, equal_1.isNotEmpty)(accessToken) ? `Bearer ${accessToken}` : undefined;
            // 如果不是 RefreshToken API 但又是在進行刷新中，那就儲存請求並取消這次發送
            if (!(0, utils_2.checkIsRefreshTokenAPI)(originConfig) && isTokenRefreshing) {
                // 將尚未送出的請求存入貯列
                pushPendingRequestQueues(resolve, reject)(originConfig);
                reject(new AxiosCancelException_1.default({ message: 'Token refreshing, so request save queues not send', code: 'REFRESH_TOKEN' }));
            }
            resolve(originConfig);
        });
    };
    /**
     * 回應攔截器
     * @param response
     */
    const interceptorsResponse = (response) => {
        const originalConfig = response.config;
        const error = (0, utils_2.getResponseFirstError)(response);
        const { refreshToken } = (0, utils_1.getTokenInfo)();
        // 判定為錯誤
        if (error) {
            if (['WS_FORBIDDEN'].includes(error.code)) {
                // 無工作區權限
                dispatch(auth_1.authActions.changeWorkspace({ workspaceId: 'me' }));
                navigate('/fe/dashboard', { replace: true });
            }
            if (error.code === 'UNAUTHENTICATED') {
                // 如果錯誤代碼為 401 UNAUTHENTICATED 但 RefreshToken 又不存在的話，直接登出
                // 如果是 Refresh Token API 失敗，則登出，並取消所有貯列請求
                if ((0, equal_1.isEmpty)(refreshToken) || (0, utils_2.checkIsRefreshTokenAPI)(originalConfig)) {
                    KitOut.mutate();
                    return Promise.reject((0, utils_2.getSystemError)(response));
                }
                // 如果不是在 Refreshing Token，則發出 Refresh Token 請求
                if (!isTokenRefreshing) {
                    isTokenRefreshing = true;
                    postRefreshToken();
                }
                return new Promise((resolve, reject) => {
                    // 將已經發送出去的請求存入貯列
                    pushPendingRequestQueues(resolve, reject)(originalConfig);
                });
            }
            return Promise.reject((0, utils_2.getSystemError)(response));
        }
        return response;
    };
    return <exports.AxiosClientContext.Provider value={config_1.axiosInstance}>
        {children}
    </exports.AxiosClientContext.Provider>;
};
exports.default = AxiosClientProvider;
//# sourceMappingURL=AxiosClientProvider.js.map