import { IUseFetcherArgs } from './fetcher';
export declare const placeholderData: any;
/**
 * 產生 useQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
export declare const createQueryHook: <TData, TVariables extends {}, TError = unknown>(document: string, queryKey: string) => (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData, TError, TData>) => any;
/**
 * 產生 queryClient.fetchQuery 的工廠方法
 * @param document
 * @param queryKey
 */
export declare const createQueryClient: <TData, TVariables extends {}, TError = unknown>(document: string, queryKey: string) => () => {
    getQueryKey: (variables?: TVariables) => (string | Object)[];
    fetchQuery: (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData, TError, TData>) => Promise<any>;
    reFetchQuery: (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData, TError, TData>) => Promise<any>;
    setQueryData: (variables: TVariables, updater: Updater<any, any>) => any;
    invalidateQueries: (args?: IUseFetcherArgs<TVariables>) => Promise<any>;
};
/**
 * 產生 useQuery & useFetchQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
export declare const createQueryAndQueryClientHook: <TData, TVariables extends {}, TError = unknown>(document: string, queryKey: string) => {
    useQuery: (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData_1, TError_1, TData_1>) => any;
    useQueryClient: () => {
        getQueryKey: (variables?: TVariables) => (string | Object)[];
        fetchQuery: (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData_2, TError_2, TData_2>) => Promise<any>;
        reFetchQuery: (args?: IUseFetcherArgs<TVariables>, options?: UseQueryOptions<TData_2, TError_2, TData_2>) => Promise<any>;
        setQueryData: (variables: TVariables, updater: Updater<any, any>) => any;
        invalidateQueries: (args?: IUseFetcherArgs<TVariables>) => Promise<any>;
    };
};
/**
 * 產生 useInfiniteQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
export declare const createInfiniteQueryHook: <TData, TVariables extends {}, TError = unknown>(document: string, queryKey: string) => (initialPageParam: number, getNextPageParam: (lastPage: TData) => number | void, args?: IUseFetcherArgs<TVariables>, options?: UseInfiniteQueryOptions<TData, TError, {
    pages: TData[];
    pageParams: any;
}>) => any;
/**
 * 產生 useMutation Hook 的工廠方法
 * @param document
 * @param mutationKey
 */
export declare const createMutationHook: <TData, TArgs extends {}, TError = unknown, TContext = unknown>(document: string, mutationKey: string) => (options?: UseMutationOptions<TData, TError, IUseFetcherArgs<TArgs>, TContext>) => any;
