"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMutationHook = exports.createInfiniteQueryHook = exports.createQueryAndQueryClientHook = exports.createQueryClient = exports.createQueryHook = exports.placeholderData = void 0;
const react_locale_1 = require("@acrool/react-locale");
const react_query_1 = require("@tanstack/react-query");
const fetcher_1 = require("./fetcher");
// Query 不同key資料的時候，會保留上一個 queryKey的資料等待覆蓋
exports.placeholderData = react_query_1.keepPreviousData;
/**
 * 產生QueryKey
 * @param rootKey
 * @param args
 */
const generateQueryKey = (rootKey, args) => {
    return [rootKey, args !== null && args !== void 0 ? args : {}];
};
/**
 * 產生 useQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
const createQueryHook = (document, queryKey // 需要傳入對應的 query key
) => (args, options) => {
    const fetcher = (0, fetcher_1.useFetchData)(document);
    // const {locale} = useLocale();
    // const localeArgs = args?.fetchOptions?.ignoreLocale ? undefined: locale;
    const localeArgs = 'en-US';
    return (0, react_query_1.useQuery)(Object.assign({ queryKey: generateQueryKey(queryKey, Object.assign(Object.assign({}, args === null || args === void 0 ? void 0 : args.variables), { localeArgs })), queryFn: () => fetcher(args) }, options));
};
exports.createQueryHook = createQueryHook;
/**
 * 產生 queryClient.fetchQuery 的工廠方法
 * @param document
 * @param queryKey
 */
const createQueryClient = (document, queryKey // 需要傳入對應的 query key
) => () => {
    const qc = (0, react_query_1.useQueryClient)();
    const { locale } = (0, react_locale_1.useLocale)();
    const fetcher = (0, fetcher_1.useFetchData)(document);
    /**
     * 取得 QueryKey
     * @param variables
     */
    const getQueryKey = (variables) => {
        // const localeArgs = args?.fetchOptions?.ignoreLocale ? undefined: locale;
        const localeArgs = 'en-US';
        return generateQueryKey(queryKey, Object.assign(Object.assign({}, variables), { localeArgs }));
    };
    /**
 * 取得 Query Data
 * (優先使用Cache)
 * @param args
 * @param options
 */
    const fetchQuery = (args, options) => __awaiter(void 0, void 0, void 0, function* () {
        return qc.fetchQuery(Object.assign({ queryKey: getQueryKey(args === null || args === void 0 ? void 0 : args.variables), queryFn: () => fetcher(args) }, options));
    });
    /**
 * 重新拿取的 QueryData
 * @param args
 * @param options
 */
    const reFetchQuery = (args, options) => __awaiter(void 0, void 0, void 0, function* () {
        yield invalidateQueries(args);
        return fetchQuery(args, options);
    });
    /**
     * 設定 QueryData
     * @param variables
     * @param updater
     */
    const setQueryData = (variables, updater) => {
        return qc.setQueryData(getQueryKey(variables), updater);
    };
    /**
 * 過期 QueryData
 * @param args
 */
    const invalidateQueries = (args) => __awaiter(void 0, void 0, void 0, function* () {
        return qc.invalidateQueries({ queryKey: getQueryKey(args === null || args === void 0 ? void 0 : args.variables) });
    });
    return {
        getQueryKey,
        fetchQuery,
        reFetchQuery,
        setQueryData,
        invalidateQueries,
    };
};
exports.createQueryClient = createQueryClient;
/**
 * 產生 useQuery & useFetchQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
const createQueryAndQueryClientHook = (document, queryKey // 需要傳入對應的 query key
) => ({
    useQuery: (0, exports.createQueryHook)(document, queryKey),
    useQueryClient: (0, exports.createQueryClient)(document, queryKey),
});
exports.createQueryAndQueryClientHook = createQueryAndQueryClientHook;
/**
 * 產生 useInfiniteQuery Hook 的工廠方法
 * @param document
 * @param queryKey
 */
const createInfiniteQueryHook = (document, queryKey // 需要傳入對應的 query key
) => (initialPageParam, getNextPageParam, // lastPage 代表上一個 page 的數據
args, options) => {
    const fetcher = (0, fetcher_1.useFetchData)(document);
    const { locale } = (0, react_locale_1.useLocale)();
    // const localeArgs = args?.fetchOptions?.ignoreLocale ? undefined: locale;
    const localeArgs = 'en-US';
    return (0, react_query_1.useInfiniteQuery)(Object.assign({ queryKey: generateQueryKey(queryKey, Object.assign(Object.assign({}, args === null || args === void 0 ? void 0 : args.variables), { localeArgs })), queryFn: ({ pageParam = initialPageParam }) => fetcher(Object.assign(Object.assign({}, args), { variables: Object.assign(Object.assign({}, args === null || args === void 0 ? void 0 : args.variables), { page: pageParam }) })), initialPageParam,
        getNextPageParam }, options));
};
exports.createInfiniteQueryHook = createInfiniteQueryHook;
/**
 * 產生 useMutation Hook 的工廠方法
 * @param document
 * @param mutationKey
 */
const createMutationHook = (document, mutationKey) => (options) => (0, react_query_1.useMutation)(Object.assign({ mutationKey: [mutationKey], mutationFn: (0, fetcher_1.useFetchData)(document) }, options));
exports.createMutationHook = createMutationHook;
//# sourceMappingURL=createQueryHookFactory.js.map