"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFetchData = void 0;
const AxiosClientProvider_1 = require("../axios/AxiosClientProvider");
const utils_1 = require("./utils");
// doc: https://the-guild.dev/graphql/codegen/plugins/typescript/typescript-react-query#usage-example-isreacthook-true
const delay = (ms) => new Promise(resolve => {
    setTimeout(() => {
        resolve(true);
    }, ms);
});
const useFetchData = (query) => {
    const axiosInstance = (0, AxiosClientProvider_1.useAxiosClient)();
    return (args) => __awaiter(void 0, void 0, void 0, function* () {
        let data = undefined;
        let contentType = undefined;
        const options = args === null || args === void 0 ? void 0 : args.fetchOptions;
        const variables = args === null || args === void 0 ? void 0 : args.variables;
        let isMultipartFormData = false;
        if (variables) {
            const varOptions = (0, utils_1.getVariablesFileMap)(variables);
            isMultipartFormData = varOptions.values.length > 0;
            // 如有檔案上傳，變更格式
            if (isMultipartFormData) {
                contentType = 'multipart/form-data';
                const operations = JSON.stringify({
                    query,
                    variables: varOptions.variables
                });
                const graphqlFormOptions = [
                    { name: 'operations', value: operations },
                    { name: 'map', value: JSON.stringify({ 0: varOptions.map }) },
                    ...varOptions.values.map((row, index) => {
                        return { name: index, value: row };
                    }),
                ];
                data = new FormData();
                graphqlFormOptions.forEach(row => {
                    data.append(row.name.toString(), row.value);
                });
            }
        }
        if (!isMultipartFormData) {
            contentType = 'application/json';
            data = JSON.stringify({ query, variables });
        }
        const endpoint = '/';
        const headers = {
            'Content-Type': contentType,
            'Apollo-Require-Preflight': 'true',
            'X-Requested-With': 'XMLHttpRequest',
        };
        const [res] = yield Promise.all([
            axiosInstance.post(endpoint, data, Object.assign(Object.assign({}, options), { headers })),
            delay(400),
        ]);
        return res.data.data;
    });
};
exports.useFetchData = useFetchData;
//# sourceMappingURL=fetcher.js.map