"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVariablesFileMap = void 0;
const isFile = input => 'File' in window && input instanceof File;
const isBlob = input => 'Blob' in window && input instanceof Blob;
/**
 * 處理表單送出時, 將 data object 轉換成 graphql upload 格式
 * @param originVariables
 * @param parentKey
 */
const getVariablesFileMap = (originVariables, parentKey = ['variables']) => {
    return originVariables && Object.keys(originVariables).reduce((curr, key) => {
        const row = originVariables[key];
        if (isFile(row) || isBlob(row)) {
            const file = row;
            return {
                variables: Object.assign(Object.assign({}, curr.variables), { [key]: null }),
                map: [...curr.map, parentKey.concat(key).join('.')],
                values: [...curr.values, file],
            };
        }
        else if (row && typeof row === 'object') {
            // 如果是Object, 往下繼續帶上層Key
            const children = (0, exports.getVariablesFileMap)(row, parentKey.concat(key));
            return {
                variables: Object.assign(Object.assign({}, curr.variables), { [key]: children.variables }),
                map: [...curr.map, ...children.map],
                values: [...curr.values, ...children.values],
            };
        }
        return Object.assign(Object.assign({}, curr), { variables: Object.assign(Object.assign({}, curr.variables), { [key]: row }), values: curr.values });
    }, { variables: {}, map: [], values: [] });
};
exports.getVariablesFileMap = getVariablesFileMap;
//# sourceMappingURL=utils.js.map